@@ -6,15 +6,20 @@
 
 #include <QDBusConnection>
 #include <QDebug>
+#include <QQmlEngine>
 
+#include <KLocalizedString>
 #include <KWindowSystem>
+#include <KX11Extras>
 
+#include <PolkitQt1/ActionDescription>
 #include <PolkitQt1/Agent/Session>
 #include <PolkitQt1/Details>
 #include <PolkitQt1/Identity>
 #include <PolkitQt1/Subject>
 
-#include "AuthDialog.h"
+#include "IdentitiesModel.h"
+#include "QuickAuthDialog.h"
 #include "policykitlistener.h"
 #include "polkit1authagentadaptor.h"
 
@@ -32,6 +37,9 @@ PolicyKitListener::PolicyKitListener(QObject *parent)
         qWarning() << "Could not initiate DBus helper!";
     }
 
+    qmlRegisterType<IdentitiesModel>("org.kde.polkitkde", 1, 0, "IdentitiesModel");
+    qmlRegisterUncreatableType<PolkitQt1::ActionDescription>("org.kde.polkitkde", 1, 0, "ActionDescription", "nope!");
+
     qDebug() << "Listener online";
 }
 
@@ -53,6 +61,10 @@ void PolicyKitListener::initiateAuthentication(const QString &actionId,
                                                const PolkitQt1::Identity::List &identities,
                                                PolkitQt1::Agent::AsyncResult *result)
 {
+    // The auth action might set any random old icon; all we really want here is a nice
+    // generic "lock" icon, so we hardcode it and ignore the icon from the auth action.
+    Q_UNUSED(iconName);
+
     qDebug() << "Initiating authentication";
 
     if (m_inProgress) {
@@ -71,16 +83,14 @@ void PolicyKitListener::initiateAuthentication(const QString &actionId,
 
     const WId parentId = m_actionsToWID.value(actionId, 0);
 
-    m_dialog = new AuthDialog(actionId, message, iconName, details, identities, parentId);
+    m_dialog = new QuickAuthDialog(actionId, message, details, identities, parentId);
     connect(m_dialog.data(), SIGNAL(okClicked()), SLOT(dialogAccepted()));
     connect(m_dialog.data(), SIGNAL(rejected()), SLOT(dialogCanceled()));
-    connect(m_dialog.data(), SIGNAL(adminUserSelected(PolkitQt1::Identity)), SLOT(userSelected(PolkitQt1::Identity)));
 
-    qDebug() << "WinId of the dialog is " << m_dialog.data()->winId() << m_dialog.data()->effectiveWinId();
-    m_dialog.data()->setOptions();
-    m_dialog.data()->show();
-    KWindowSystem::forceActiveWindow(m_dialog.data()->winId());
-    qDebug() << "WinId of the shown dialog is " << m_dialog.data()->winId() << m_dialog.data()->effectiveWinId();
+    m_dialog->show();
+    if (KWindowSystem::isPlatformX11()) {
+        KX11Extras::forceActiveWindow(m_dialog->windowHandle()->winId());
+    }
 
     if (identities.length() == 1) {
         m_selectedUser = identities[0];
@@ -166,10 +176,6 @@ void PolicyKitListener::request(const QString &request, bool echo)
 {
     Q_UNUSED(echo);
     qDebug() << "Request: " << request;
-
-    if (!m_dialog.isNull()) {
-        m_dialog.data()->setRequest(request, m_selectedUser.isValid() && m_selectedUser.toString() == "unix-user:root");
-    }
 }
 
 void PolicyKitListener::completed(bool gainedAuthorization)
@@ -201,7 +207,7 @@ void PolicyKitListener::dialogAccepted()
 {
     qDebug() << "Dialog accepted";
 
-    if (!m_dialog.isNull()) {
+    if (!m_session.isNull() && !m_dialog.isNull()) {
         m_session.data()->setResponse(m_dialog.data()->password());
     }
 }
@@ -227,3 +233,5 @@ void PolicyKitListener::userSelected(const PolkitQt1::Identity &identity)
     }
     tryAgain();
 }
+
+#include "moc_policykitlistener.cpp"
